local data = {}
for i = 1, 20 do
	table.insert(data, ""..i) 
end

local m_touchPoint = nil
local m_nCurPage = 1


-- @param fn string Callback type
-- @param table LuaTableView
-- @param a1 & a2 mixed Difference means for every "fn"
local h = LuaEventHandler:create(function(fn, table, a1, a2)
	local r
	if fn == "cellSize" then
		-- Return cell size
		-- a1 is cell index (-1 means default size, in cocos2d-x version below 2.1.3, it's always -1)
		r = CCSizeMake(60,60)
	elseif fn == "cellAtIndex" then
		-- Return CCTableViewCell, a1 is cell index (zero based), a2 is dequeued cell (maybe nil)
		-- Do something to create cell and change the content
		if not a2 then
			a2 = CCTableViewCell:create()
			-- Build cell struct, just like load ccbi or add sprite
			local sprite = CCSprite:create("icon.png")
			sprite:setAnchorPoint(ccp(0, 0))
			sprite:setPosition(ccp(0, 0))
			a2:addChild(sprite)

			local label = CCLabelTTF:create("", "Arial", 20)
			label:setAnchorPoint(ccp(0, 0))
			label:setPosition(ccp(0, 0))
			label:setString(data[a1 + 1])
			a2:addChild(label)
		end

		r = a2
	elseif fn == "numberOfCells" then
		-- Return number of cells
		r = #data
	-- Cell events:
	elseif fn == "cellTouched" then			-- A cell was touched, a1 is cell that be touched. This is not necessary.
		CCLuaLog("cellTouched at index: " .. (a1:getIdx() + 1))
	elseif fn == "cellTouchBegan" then		-- A cell is touching, a1 is cell, a2 is CCTouch
		m_touchPoint = CCDirector:sharedDirector():convertToGL(a2:getLocationInView())
		r = true
	elseif fn == "cellTouchEnded" then		-- A cell was touched, a1 is cell, a2 is CCTouch
		local endPoint = CCDirector:sharedDirector():convertToGL(a2:getLocationInView())
		local distance = endPoint.x - m_touchPoint.x
		CCLuaLog("distance = " .. distance)
		--adjustTableView(distance, a1)
		if math.abs(distance) > 10 then
			--adjustTableView(distance, a1)
		end
	elseif fn == "cellHighlight" then		-- A cell is highlighting, coco2d-x 2.1.3 or above
	elseif fn == "cellUnhighlight" then		-- A cell had been unhighlighted, coco2d-x 2.1.3 or above
	elseif fn == "cellWillRecycle" then		-- A cell will be recycled, coco2d-x 2.1.3 or above
	end
	return r
end)

local t = LuaTableView:createWithHandler(h, CCSizeMake(300,60))
t:setDirection(kCCScrollViewDirectionHorizontal)
t:setVerticalFillOrder(kCCTableViewFillTopDown)
t:reloadData()
t:setBounceable(true)
t:setAnchorPoint(ccp(0,0))
t:setPosition(ccp(20, 100))

--[[
CCLuaLog("contentSize w: " .. t:getContentSize().width)
CCLuaLog("contentSize h: " .. t:getContentSize().height)

CCLuaLog("viewSize    w: " .. t:getViewSize().width)
CCLuaLog("viewSize    h: " .. t:getViewSize().height)
--]]

function adjustTableView(offset, cell)
	if offset < 0 then
		m_nCurPage = m_nCurPage + 1
	else
		m_nCurPage = m_nCurPage - 1
	end

	if m_nCurPage < 1 then
		m_nCurPage = 1
	end

	CCLuaLog("m_nCurPage = "..m_nCurPage)

	if  m_nCurPage > 20 then
		CCLuaLog("m_nCurPage > 20, m_nCurPage = "..m_nCurPage)
	else
		local originPoint = t:getPositionX()
		CCLuaLog("originPoint = " .. originPoint)
		CCLuaLog("a = " .. cell:getContentSize().width)
		CCLuaLog("b = " .. (cell:getContentSize().width * (m_nCurPage-1)))
		local adjustPoint = ccp(originPoint - 60 * (m_nCurPage-1), t:getPositionY())
		CCLuaLog("adjustPoint = " .. adjustPoint.x)
		t:setContentOffset(adjustPoint)
	end
end

--[[
-- Temporary unusable, please skip these lines to the comment section's end.
-- Add scroll bar and scroll track (track is optional), they'll be placed right or bottom in table.
-- The function LuaTableView:setScrollOffset(float) can be used for adjust scroll bar's position (bigger to right or bottom)
-- Assume those two sprite frames were loaded.
local sfc = CCSpriteFrameCache:sharedSpriteFrameCache()
local bar = CCScale9Sprite:createWithSpriteFrame(sfc:spriteFrameByName("scrollBar.png"))
bar:setOpacity(80)
bar:setPreferredSize(CCSizeMake(10, 10))
local trk = CCScale9Sprite:createWithSpriteFrame(sfc:spriteFrameByName("scrollTrack.png"))
trk:setOpacity(128)
trk:setPreferredSize(CCSizeMake(15, 15))
t:setScrollBar(bar, trk)
]]

-- Create a scene and run
local l = CCLayer:create()
l:addChild(t)
local s = CCScene:create()
s:addChild(l)
CCDirector:sharedDirector():runWithScene(s)
